###############################################################################
#
# Copyright (C) 2014
# ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from common import *

from mem_init_file import list_to_hex
import matplotlib
matplotlib.use('TkAgg')
from common_dsp import *
import matplotlib.pyplot as plt

import numpy as np
from scipy.fftpack import fft,ifft, fftfreq, fftshift

# Purpose:
# . Generate the NOF_INPUTS HEX files for the block generators that feed the WPFB
# Description:
# . 

NOF_INPUTS = 12
NOF_INPUT_SIGNALS = 24
COMPLEX_WIDTH = 8
NOF_WORDS_PER_BLOCK = 64

PATH = "../hex"
FILENAME = "composite_signals"

###############################################################################
# Create composite waves
###############################################################################
N = NOF_WORDS_PER_BLOCK
t=range(N)

# sample spacing
T = 1.0 / N

x = np.linspace(0.0, N*T, N)

# 8 bit signed -> WAAROM LIJKT AMPL_MAX DAN TOCH 63 te zijn IPV 127??
# . Boven ~70 wordt de waarde aan de FFT output jusit lager...
AMPL_MAX = 63 # 7 bits+sign #HUH?! 100->62; 63->118; 80->105; 70->123; 72->121; 75->110
NOF_CHANNELS = 1 # Note: starting from 0 in -31..31.

input_signals = []
for input_signal_nr in range(NOF_INPUT_SIGNALS):
    phase_shift_deg = input_signal_nr
    channel_signals = []
    for bin_nr in range(NOF_CHANNELS):
        phase_shift_rad = math.radians(phase_shift_deg)
        ampl=AMPL_MAX/NOF_CHANNELS
        signal = ampl * np.exp( (bin_nr+1) * 1.j * (2.0*np.pi*x+phase_shift_rad) )
        channel_signals.append( signal )
    composite_signal=numpy.sum(channel_signals, axis=0)
    input_signals.append(composite_signal)

s=input_signals[23]
###############################################################################
# Convert the float values to n-bits
###############################################################################
#s_bits = []
#for fword in s:
#    re_signed = to_signed(fword.real, COMPLEX_WIDTH)
#    im_signed = to_signed(fword.imag, COMPLEX_WIDTH)
#
#    s_bits.append( complex(re_signed, im_signed) )
#
#s = numpy.array(s_bits)
#print s

###############################################################################
# Plot the signal
###############################################################################
#plt.plot(t, s.real, 'b-', t, s.imag, 'r--')
#plt.legend(('real', 'imaginary'))
#plt.show()



###############################################################################
# Plot FFT
###############################################################################
#yf = fft(s)
#
#xf = fftfreq(N, T)
#xf = fftshift(xf)
#yplot = fftshift(yf)
#plt.bar(xf, 1.0/N * np.abs(yplot))
#plt.grid()
#plt.show()

###############################################################################
# Convert complex floats to concatenated integers
###############################################################################
#concat_list = concat_complex(input_signals[0], COMPLEX_WIDTH)

input_signals_concat = []
for input_signal in input_signals:
    input_signals_concat.append( concat_complex(input_signal, COMPLEX_WIDTH) )

###############################################################################
# Interleave 2 lists into one
###############################################################################
#inter_list = interleave([concat_list,concat_list])

input_signals_concat_inter = []
for i in range(NOF_INPUTS):
    input_signals_concat_inter.append( interleave([input_signals_concat[2*i],input_signals_concat[2*i+1]]) )

###############################################################################
# Use this list for each block generator
###############################################################################
#bg_lists = NOF_INPUTS*[inter_list]
bg_lists = input_signals_concat_inter

###############################################################################
# Write the HEX files
###############################################################################
MEM_WIDTH  = COMPLEX_WIDTH*2
MEM_DEPTH  = 2*NOF_WORDS_PER_BLOCK # We're interleaving, hence twice the depth

for input_nr in range(NOF_INPUTS):
    list_to_hex( bg_lists[input_nr], PATH+"/"+FILENAME+'_'+str(input_nr)+".hex", MEM_WIDTH, MEM_DEPTH)

