-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, dp_lib, bf_lib, unb_common_lib, tse_lib, diag_lib, fn_bf_lib, fn_terminal_db_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE bf_lib.bf_pkg.ALL;
USE tse_lib.tse_pkg.ALL;
USE unb_common_lib.unb_common_pkg.ALL;
USE tse_lib.eth_layers_pkg.ALL;
USE tse_lib.eth_pkg.ALL;
b
hdl_lib_uses = common technology m
ENTITY node_fn_beamformer IS
  GENERIC(
    g_sim                     : BOOLEAN := FALSE;
    g_sim_level               : NATURAL := 0;
    g_use_data_buf            : BOOLEAN := TRUE;  -- Add a data buffer parallel to the BF.  Does not affect other functions.
    g_use_bf_offload          : BOOLEAN := TRUE;  -- Offload BF out(0) datapath to 1GbE UDP TX port. Does not affect other functions.
    g_use_bf                  : BOOLEAN := TRUE;  -- FALSE does not instantiate a BF instance.
    g_use_block_gen           : BOOLEAN := FALSE; -- TRUE overrides terminal output and feeds BG output to the BF instead.
    -- Stream properties 
    g_nof_streams             : NATURAL := 16;
--    g_data_w                  : NATURAL := 32;

    g_bf                      : t_c_bf   := c_bf;
    g_bf_weights_file_name    : STRING   := "../../../../../dsp/bf/build/data/weights"; -- default file location for synthesis
    -- Auxiliary Interface
    g_aux                     : t_c_unb_aux := c_unb_aux
  );
  PORT(
    -- System
    chip_id                     : IN  STD_LOGIC_VECTOR(g_aux.chip_id_w-1 DOWNTO 0);  -- [2:0]
    
    mm_rst                      : IN  STD_LOGIC;
    mm_clk                      : IN  STD_LOGIC;   -- 50 MHz from xo_clk PLL in SOPC system
    dp_rst                      : IN  STD_LOGIC;
    dp_clk                      : IN  STD_LOGIC;   -- 200 MHz from CLK system clock
    dp_pps                      : IN  STD_LOGIC := '0';
    tr_mesh_clk                 : IN  STD_LOGIC;   -- 156.25 MHz from SA_CLK transceiver clock
    cal_clk                     : IN  STD_LOGIC;   -- 40 MHz from xo_clk PLL in SOPC system

    -- MM interface
    -- . block generator
    reg_diag_bg_mosi            : IN  t_mem_mosi; 
    reg_diag_bg_miso            : OUT t_mem_miso; 
    ram_diag_bg_mosi            : IN  t_mem_mosi; 
    ram_diag_bg_miso            : OUT t_mem_miso; 
    -- . beam former
    ram_ss_ss_wide_mosi         : IN  t_mem_mosi;
    ram_ss_ss_wide_miso         : OUT t_mem_miso;  
    ram_bf_weights_mosi         : IN  t_mem_mosi;
    ram_bf_weights_miso         : OUT t_mem_miso; 
    ram_st_sst_bf_mosi          : IN  t_mem_mosi;
    ram_st_sst_bf_miso          : OUT t_mem_miso;
    reg_st_sst_bf_mosi          : IN  t_mem_mosi;
    reg_st_sst_bf_miso          : OUT t_mem_miso;
    -- . hdr_insert for bf_out_offload
    reg_hdr_insert_mosi         : IN  t_mem_mosi := c_mem_mosi_rst;
    ram_hdr_insert_mosi         : IN  t_mem_mosi := c_mem_mosi_rst;
    -- . dp_split for bf_out_offload
    reg_dp_split_mosi           : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_dp_split_miso           : OUT t_mem_miso;
    reg_dp_pkt_merge_mosi       : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_dp_pkt_merge_miso       : OUT t_mem_miso;
    -- . diag_data_buffer
    ram_diag_data_buf_mosi      : IN  t_mem_mosi := c_mem_mosi_rst;
    ram_diag_data_buf_miso      : OUT t_mem_miso;
    reg_diag_data_buf_mosi      : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_diag_data_buf_miso      : OUT t_mem_miso;
    -- . tr_nonbonded
    reg_tr_nonbonded_mosi       : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_tr_nonbonded_miso       : OUT t_mem_miso;
    reg_diagnostics_mosi        : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_diagnostics_miso        : OUT t_mem_miso;
    -- . bsn_monitor
    reg_bsn_monitor_mosi        : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_bsn_monitor_miso        : OUT t_mem_miso;

    -- ST interface, BF beamlets out
    beamlets_bst_sosi_arr       : OUT t_dp_sosi_arr(g_bf.nof_bf_units-1 DOWNTO 0);  -- 16b beamlets; a selection can be offloaded via bf_out_offload_tx_sosi_arr.
    beamlets_qua_sosi_arr       : OUT t_dp_sosi_arr(g_bf.nof_bf_units-1 DOWNTO 0);  --  8b beamlets
    
    -- DP offload for 1GbE
    bf_out_offload_tx_sosi_arr  : OUT t_dp_sosi_arr(c_eth_nof_udp_ports-1 DOWNTO 0);  -- 16b beamlets
    bf_out_offload_tx_siso_arr  : IN  t_dp_siso_arr(c_eth_nof_udp_ports-1 DOWNTO 0) := (OTHERS => c_dp_siso_rst);

    -- Mesh interface level
    -- . Serial (tr_nonbonded)
    tx_serial_2arr              : OUT t_unb_mesh_sl_2arr;                            -- Tx
    rx_serial_2arr              : IN  t_unb_mesh_sl_2arr:= (OTHERS=>(OTHERS=>'0'))   -- Rx support for diagnostics
  );
END node_fn_beamformer;

ARCHITECTURE str OF node_fn_beamformer IS

  CONSTANT c_weights_write_only     : BOOLEAN := TRUE;
  -- Use default RAM init files. The RAM init file for simulation lies one ../ level further way then for synthesis
  CONSTANT c_bf_weights_file_name   : STRING := sel_a_b(g_sim, "../", "") & "../../../../../dsp/bf/build/data/weights";
  CONSTANT c_block_gen_file_prefix  : STRING := sel_a_b(g_sim, "../", "") & "../../../../../modules/Lofar/diag/src/data/bf_in_data";
  CONSTANT c_ss_wide_file_prefix    : STRING  := "UNUSED"; -- path_to_file    

  SIGNAL node_fn_term_out_sosi_arr  : t_dp_sosi_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL node_fn_term_out_siso_arr  : t_dp_siso_arr(g_nof_streams-1 DOWNTO 0);
  SIGNAL node_bf_in_sosi_arr        : t_dp_sosi_arr( g_bf.nof_input_streams-1 DOWNTO 0); 
  SIGNAL node_bf_in_siso_arr        : t_dp_siso_arr( g_bf.nof_input_streams-1 DOWNTO 0); 
  
BEGIN

  -----------------------------------------------------------------------------
  -- Node function: Beam Former + BF UDP offload + block generator
  -----------------------------------------------------------------------------  
  u_node_fn_bf : ENTITY fn_bf_lib.node_fn_bf
  GENERIC MAP(
    g_bf                    => c_bf,
    g_bf_offload            => g_use_bf_offload,
    g_use_bf                => g_use_bf,
    g_use_block_gen         => g_use_block_gen,
    g_bf_weights_file_name  => c_bf_weights_file_name,
    g_ss_wide_file_prefix   => c_ss_wide_file_prefix,  
    g_block_gen_file_prefix => c_block_gen_file_prefix,
    g_weights_write_only    => c_weights_write_only
  )
  PORT MAP(
    -- System
    mm_rst                 => mm_rst, 
    mm_clk                 => mm_clk, 
    dp_rst                 => dp_rst, 
    dp_clk                 => dp_clk, 
    -- MM interface
    -- . block generator   
    reg_diag_bg_mosi       => reg_diag_bg_mosi,  
    reg_diag_bg_miso       => reg_diag_bg_miso,
    ram_diag_bg_mosi       => ram_diag_bg_mosi,  
    ram_diag_bg_miso       => ram_diag_bg_miso,  
    -- . beam former
    ram_ss_ss_wide_mosi    => ram_ss_ss_wide_mosi,
    ram_ss_ss_wide_miso    => ram_ss_ss_wide_miso,  
    ram_bf_weights_mosi    => ram_bf_weights_mosi,   
    ram_bf_weights_miso    => ram_bf_weights_miso,   
    ram_st_sst_bf_mosi     => ram_st_sst_bf_mosi,      
    ram_st_sst_bf_miso     => ram_st_sst_bf_miso,
    reg_st_sst_bf_mosi     => reg_st_sst_bf_mosi,      
    reg_st_sst_bf_miso     => reg_st_sst_bf_miso,
    -- . hdr_insert for dp offload
    reg_hdr_insert_mosi    => reg_hdr_insert_mosi,
    ram_hdr_insert_mosi    => ram_hdr_insert_mosi,
    -- . dp_split for dp offload
    reg_dp_split_mosi      => reg_dp_split_mosi,
    reg_dp_split_miso      => reg_dp_split_miso,
    reg_dp_pkt_merge_mosi  => reg_dp_pkt_merge_mosi,
    reg_dp_pkt_merge_miso  => reg_dp_pkt_merge_miso,
    -- ST interface
    ext_in_sosi_arr        => node_bf_in_sosi_arr,
    ext_in_siso_arr        => node_bf_in_siso_arr,
    out_bst_sosi_arr       => beamlets_bst_sosi_arr,  -- 16b beamlets; a selection can be offloaded via bf_out_offload_tx_sosi_arr.
    out_qua_sosi_arr       => beamlets_qua_sosi_arr,  --  8b beamlets

    -- DP offload for 1GbE
    bf_out_offload_tx_sosi_arr => bf_out_offload_tx_sosi_arr,  -- 16b beamlets
    bf_out_offload_tx_siso_arr => bf_out_offload_tx_siso_arr
  );

  -----------------------------------------------------------------------------
  -- Interconnect node functions: terminal output to beamformer input
  -----------------------------------------------------------------------------  
  node_bf_in_sosi_arr       <= node_fn_term_out_sosi_arr;
  node_fn_term_out_siso_arr <= node_bf_in_siso_arr;

  -----------------------------------------------------------------------------
  -- Node function: Terminals + data buffer
  -----------------------------------------------------------------------------  
  u_node_fn_terminal_db : ENTITY fn_terminal_db_lib.node_fn_terminal_db
  GENERIC MAP(
    g_sim                       => g_sim,
    g_sim_level                 => g_sim_level,
    g_use_data_buf              => g_use_data_buf,
    -- Terminals interface
    g_multi_unb                 => TRUE,
    -- Auxiliary Interface
    g_aux                       => c_unb_aux
  )
  PORT MAP(
    -- System
    mm_rst                      => mm_rst,
    mm_clk                      => mm_clk,
    dp_rst                      => dp_rst,
    dp_clk                      => dp_clk,
    dp_pps                      => dp_pps,
    tr_mesh_clk                 => tr_mesh_clk,
    cal_clk                     => cal_clk,

    chip_id                     => chip_id,

    -- MM interface
    -- . tr_nonbonded
    reg_tr_nonbonded_mosi       => reg_tr_nonbonded_mosi,
    reg_tr_nonbonded_miso       => reg_tr_nonbonded_miso,
    reg_diagnostics_mosi        => reg_diagnostics_mosi,
    reg_diagnostics_miso        => reg_diagnostics_miso,
    -- . diag_data_buffer
    ram_diag_data_buf_mosi      => ram_diag_data_buf_mosi,
    ram_diag_data_buf_miso      => ram_diag_data_buf_miso,
    reg_diag_data_buf_mosi      => reg_diag_data_buf_mosi,
    reg_diag_data_buf_miso      => reg_diag_data_buf_miso,
    -- . bsn_monitor
    reg_bsn_monitor_mosi        => reg_bsn_monitor_mosi,
    reg_bsn_monitor_miso        => reg_bsn_monitor_miso,
    
    -- Mesh interface
    tx_serial_2arr              => tx_serial_2arr,
    rx_serial_2arr              => rx_serial_2arr,

    dp_out_sosi_arr             => node_fn_term_out_sosi_arr,
    dp_out_siso_arr             => node_fn_term_out_siso_arr    
  );

END str;

